from typing import Any

from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.postgres.expressions import ArraySubquery
from django.db import IntegrityError
from django.db import models
from django.db import ProgrammingError
from django.db.models import OuterRef
from django.db.models import Q
from django.db.models import QuerySet
from django.db.models import Subquery
from django.db.models.expressions import RawSQL
from django.db.models.functions import Cast
from django.db.models.functions import JSONObject
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel

from django_features.custom_fields.models.field import CustomField
from django_features.custom_fields.models.field import CustomFieldQuerySet
from django_features.custom_fields.models.value import CustomValue


class CustomFieldModelBaseManager(models.Manager):
    def get_type_model(self) -> "CustomFieldTypeBaseModel | None":
        if self.model._custom_field_type_attr is None or not hasattr(
            self.model, self.model._custom_field_type_attr
        ):
            return None
        return getattr(
            self.model, self.model._custom_field_type_attr
        ).field.related_model

    def get_type_filter(self) -> Q:
        type_model = self.get_type_model()
        if type_model is not None:
            type_filter = {
                "type_content_type__app_label": type_model._meta.app_label,
                "type_content_type__model": type_model._meta.model_name,
                "type_id": OuterRef(f"{self.model._custom_field_type_attr}_id"),
            }
            return Q(**type_filter) | Q(type_id__isnull=True)
        return Q()

    def _subquery(self, field: CustomField) -> Subquery:
        pk_filter = {
            f"{self.model._meta.model_name}__id": OuterRef("pk"),
        }

        custom_values_queryset = CustomValue.objects.for_model(self.model).filter(
            **pk_filter, field__identifier=field.identifier
        )

        if not field.choice_field:
            return Subquery(
                custom_values_queryset.annotate(
                    formated=Cast(
                        (
                            RawSQL(
                                f"ARRAY(SELECT (jsonb_array_elements_text(value))::{field.sql_field})",
                                [],
                            )
                            if field.multiple
                            else RawSQL("(value #>> '{}')", [])
                        ),
                        output_field=field.output_field,
                    )
                ).values_list("formated", flat=True)
            )
        else:
            sq = ArraySubquery if field.multiple else Subquery
            return sq(
                custom_values_queryset.annotate(
                    formated=JSONObject(id="id", label="label", value="value")
                ).values_list("formated", flat=True)
            )

    def get_queryset(self) -> QuerySet:
        """
        We filter all available custom fields for the current model.
        """
        try:
            available_fields = CustomField.objects.for_model(self.model)

            """
            This for loop creates a dict with all available custom field values with a subquery for the specific object.
            The dict key is the identifier of the custom field amd the value is the custom value.
            If the object has no value for the field, it will return None.
            More information can be found in the django documentation:
            https://docs.djangoproject.com/en/5.2/ref/models/expressions/#subquery-expressions
            """
            fields = {
                field.identifier: self._subquery(field) for field in available_fields
            }

            """
            # The dict can be unpacked and used for the dynamic annotations.
            # We also annotate the available custom field identifiers as 'custom_field_keys'.
            # Therefore, we know which custom fields are available for this object.
            """
            return (
                super()
                .get_queryset()
                .annotate(**fields)
                .annotate(
                    custom_field_keys=ArraySubquery(
                        available_fields.filter(self.get_type_filter()).values_list(
                            "identifier", flat=True
                        )
                    )
                )
            )
        except (ProgrammingError, RuntimeError, IntegrityError):
            return super().get_queryset()


class CustomFieldTypeBaseModel(TimeStampedModel):
    custom_fields = GenericRelation(
        CustomField, object_id_field="type_id", content_type_field="type_content_type"
    )

    class Meta:
        abstract = True


class CustomFieldBaseModel(TimeStampedModel):
    _custom_field_type_attr: str | None = None

    custom_values = models.ManyToManyField(
        blank=True,
        to=CustomValue,
        verbose_name=_("Benutzerdefinierte Werte"),
    )
    objects = CustomFieldModelBaseManager()

    class Meta:
        abstract = True

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self._custom_values_to_save: list[CustomValue] = []
        self._custom_values_to_remove: list[CustomValue] = []

    def save(self, **kwargs: Any) -> None:
        super().save(**kwargs)  # type: ignore
        if self._custom_values_to_remove:
            self.custom_values.remove(*self._custom_values_to_remove)

        _custom_values_to_add: set[CustomValue] = set()
        existing_custom_values = self.custom_values.all()
        for value in self._custom_values_to_save:
            value.save()  # type: ignore
            if value not in existing_custom_values:
                _custom_values_to_add.add(value)
        if _custom_values_to_add:
            self.custom_values.add(*_custom_values_to_add)
        self._custom_values_to_save = []
        self._custom_values_to_remove = []

    def refresh_with_custom_fields(self) -> None:
        self.__dict__.update(self.__class__.objects.get(pk=self.pk).__dict__)

    def _create_or_update_custom_value(self, field: str, value: Any) -> None:
        try:
            value_object = self.custom_values.select_related("field").get(field=field)
        except CustomValue.DoesNotExist:
            value_object = CustomValue(field=field)
        serializer_field = value_object.field.serializer_field
        serializer_field.run_validators(value)
        value_object.value = serializer_field.to_representation(value)
        self._custom_values_to_save.append(value_object)

    def _set_choice_value(self, field: CustomField, value: Any) -> None:
        self._custom_values_to_remove.extend(CustomValue.objects.filter(field=field))
        if field.multiple:
            # We expect a list for multiple choice fields, so we must extend the list with the items of the list
            self._custom_values_to_save.extend(value)
        else:
            self._custom_values_to_save.append(value)

    def __setattr__(self, name: str, value: Any) -> None:
        if hasattr(self, "custom_field_keys") and name in self.custom_field_keys:
            field = CustomField.objects.get(identifier=name)
            if field.choice_field:
                self._set_choice_value(field, value)
            else:
                self._create_or_update_custom_value(field, value)
        super().__setattr__(name, value)

    def delete(
        self,
        using: Any | None = None,
        keep_parents: bool = False,
        delete_custom_values: bool = True,
    ) -> None:
        if delete_custom_values:
            self.custom_values.filter(field__choice_field=False).delete()
        super().delete(using, keep_parents)

    @property
    def custom_fields(self) -> CustomFieldQuerySet:
        return CustomField.objects.for_model(self.__class__)

    @property
    def custom_field_type(self) -> CustomFieldTypeBaseModel | None:
        if self._custom_field_type_attr and hasattr(self, self._custom_field_type_attr):
            return getattr(self, self._custom_field_type_attr)
        return None

    @property
    def default_custom_fields(self) -> CustomFieldQuerySet:
        return CustomField.objects.default_for(self.__class__)

    @property
    def type_custom_fields(self) -> CustomFieldQuerySet:
        if self.custom_field_type:
            return self.custom_field_type.custom_fields.all()
        return CustomField.objects.none()



from typing import Any

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class RelatedField(serializers.RelatedField):
    """
    A relation field which expects an uid to be present on the related model for lookup and representation.
    """

    default_related_field_name = "id"

    default_error_messages = {
        "does_not_exist": _("Objekt existiert nicht."),
        "incorrect_type": _("Inkorrekter Typ."),
    }

    def __init__(
        self,
        field: models.Field | None = None,
        queryset: QuerySet | None = None,
        related_field_name: str | None = None,
        creation: bool = True,
        **kwargs: Any,
    ) -> None:
        self.creation = creation
        self.field = field
        self.queryset = queryset
        self.related_field_name = related_field_name or self.default_related_field_name
        super().__init__(**kwargs)

    def get_field(self) -> models.Field:
        if self.field:
            field = self.field
        elif self.queryset:
            field = self.queryset.model._meta.get_field(self.field_name)
        elif self.parent.Meta and self.parent.Meta.model:
            field = self.parent.Meta.model._meta.get_field(self.field_name)
        else:
            raise ValidationError(
                f"No field or queryset defined for field {self.field_name} "
                f"and parent class {self.parent.__class__} has no attribute Meta."
            )
        if not field.is_relation:
            raise ValidationError(f"Field {self.field_name} is not a relation field.")
        return field

    def get_queryset(self) -> QuerySet:
        return self.queryset or self.get_field().related_model.objects

    def to_representation(self, related_obj: models.Model) -> None | str:
        """
        Get the related value of the object; format it as UID.
        """
        if not related_obj:
            return None
        related_field = getattr(related_obj, self.related_field_name, None)
        if related_field is None and not self.required:
            raise AttributeError(
                f"No field named 'UID' defined for relation to {self.field_name} on {related_obj}"
            )
        return related_field

    def to_internal_value(self, data: str) -> models.Model | None:
        """
        Assume that the model is defined on the serializer using this field.
        """
        if not data and not self.required:
            return None
        try:
            return self.get_queryset().get(**{self.related_field_name: data})
        except ObjectDoesNotExist as e:
            if self.required:
                raise ValidationError(
                    f"Object {self.get_field().related_model} with {self.related_field_name} {data} "
                    f"does not exist: {e}"
                )
            else:
                return None
        except (TypeError, ValueError) as e:
            raise ValidationError(
                f"The data {data} for model {self.get_field().related_model} has an incorrect type "
                f"{type(data).__name__}: {e}"
            )


class UUIDRelatedField(RelatedField):
    default_related_field_name = "uid"

    def to_representation(self, related_obj: models.Model) -> None | str:
        _ = super().to_representation(related_obj)
        return serializers.UUIDField().to_representation(related_obj)

    def to_internal_value(self, data: str) -> models.Model | None:
        uuid = data
        if uuid:
            uuid = serializers.UUIDField().to_internal_value(data)
        return super().to_internal_value(uuid)


class ExternalUUIDRelatedField(UUIDRelatedField):
    default_related_field_name = "external_uid"



# Generated by Django 4.2.23 on 2025-09-03 03:14
from django.core.management import call_command
from django.db import migrations
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import StateApps


def create_default_system_info_types(
    apps: StateApps, schema_editor: BaseDatabaseSchemaEditor
) -> None:
    call_command("create_default_system_message_types")


class Migration(migrations.Migration):

    dependencies = [
        ("system_message", "0001_add_system_message_model"),
    ]

    operations = [
        migrations.RunPython(
            create_default_system_info_types, migrations.RunPython.noop
        )
    ]



from rest_framework import serializers

from django_features.system_message import models


class SystemMessageTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SystemMessageType
        fields = [
            "id",
            "icon",
            "name",
        ]


class SystemMessageSerializer(serializers.ModelSerializer):
    type = SystemMessageTypeSerializer(read_only=True)
    type_id = serializers.IntegerField()

    class Meta:
        model = models.SystemMessage
        fields = [
            "background_color",
            "begin",
            "end",
            "id",
            "text",
            "text_color",
            "title",
            "type",
            "type_id",
        ]



from typing import Any

from django.contrib.contenttypes.fields import GenericRelation
from django.core.exceptions import FieldDoesNotExist
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import NOT_PROVIDED
from rest_framework.fields import empty
from rest_framework.relations import ManyRelatedField

from django_features.custom_fields.serializers import CustomFieldBaseModelSerializer
from django_features.fields import UUIDRelatedField


class BaseMappingSerializer(CustomFieldBaseModelSerializer):
    relation_separator: str = "."
    serializer_related_field = UUIDRelatedField
    serializer_related_fields: dict[str, Any] = {}

    _write_only_serializer = True

    class Meta:
        abstract = True
        fields = "__all__"
        model = None

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self._unique_choice_field = str(
            self.mapping.get("unique_choice_field", "value")
        )
        super().__init__(*args, **kwargs)
        self.exclude: list[str] = []
        self.related_fields: set[str] = set()

    @property
    def mapping(self) -> dict[str, dict[str, Any]]:
        raise NotImplementedError("Mapping must be set")

    @property
    def mapping_fields(self) -> list[str]:
        raise NotImplementedError("Mapping fields must be set")

    @property
    def model(self) -> models.Model:
        if self.Meta.model is None:
            raise ValueError("Meta.model must be set")
        return self.Meta.model

    def get_fields(self) -> dict[str, Any]:
        initial_fields = super().get_fields()
        fields: dict[str, Any] = dict()
        nested_fields: dict[str, Any] = dict()
        nested_field_fields: dict[str, list[str]] = dict()
        self.related_fields: set[str] = set()
        for internal_name in self.mapping_fields:
            if internal_name in self.exclude:
                continue
            split = internal_name.split(self.relation_separator)
            field_name = split[0]
            serializer_field = initial_fields.get(field_name)
            if serializer_field is not None and (
                field_name in self._declared_fields or len(split) == 1
            ):
                fields[field_name] = serializer_field
            else:
                try:
                    field = self.model._meta.get_field(field_name)
                except FieldDoesNotExist:
                    raise ValidationError(
                        f"Invalid field '{field_name}' for model {self.model}."
                    )
                if len(split) > 1:
                    nested_field = self.relation_separator.join(split[1:])
                    if field_name not in nested_fields:
                        nested_fields[field_name] = field
                    if field_name in nested_field_fields:
                        nested_field_fields[field_name].append(nested_field)
                    else:
                        nested_field_fields[field_name] = [nested_field]
                elif isinstance(field, GenericRelation):
                    self.related_fields.add(field_name)
                    serializer_related_field = self.serializer_related_fields.get(
                        internal_name, self.serializer_related_field
                    )
                    fields[internal_name] = ManyRelatedField(
                        child_relation=serializer_related_field(
                            field=field, required=False
                        ),
                        required=False,
                    )
        for field_name, field in nested_fields.items():
            nested_data = self.initial_data.get(field_name)
            if not isinstance(nested_data, dict):
                continue
            self.related_fields.add(field_name)
            fields[field_name] = NestedMappingSerializer(
                data=nested_data,
                exclude=[*self.exclude, self.model.__name__.lower()],
                field=field,
                nested_fields=nested_field_fields[field_name],
                parent_mapping=self.mapping,
                required=False,
            )
        missing_declared_fields = self._declared_fields.keys() - fields.keys()
        fields.update(
            {field: initial_fields[field] for field in missing_declared_fields}
        )
        self.fields = fields
        return fields

    def create(self, validated_data: dict[str, Any]) -> models.Model:
        relations_to_save: dict[str, Any] = {}
        for field in self.related_fields:
            value = validated_data.pop(field, None)
            serializer = self.fields.get(field)
            if isinstance(serializer, NestedMappingSerializer):
                serializer.is_valid(raise_exception=True)
                relations_to_save[field] = serializer.save()
            elif value is not None:
                relations_to_save[field] = value
        instance = super().create(validated_data)
        for field, value in relations_to_save.items():
            model_field = self.model._meta.get_field(field)
            if model_field.many_to_many or model_field.one_to_many:
                getattr(instance, field).set(value)
            if model_field.one_to_one or model_field.many_to_one:
                setattr(instance, field, value)
        instance.save()
        return instance

    def update(
        self, instance: models.Model, validated_data: dict[str, Any]
    ) -> models.Model:
        for field in self.related_fields:
            value = validated_data.pop(field, None)
            serializer = self.fields.get(field)
            if isinstance(serializer, NestedMappingSerializer):
                serializer.is_valid(raise_exception=True)
                value = serializer.save()
            model_field = self.model._meta.get_field(field)
            if model_field.many_to_many or model_field.one_to_many:
                getattr(instance, field).set(value)
            if model_field.one_to_one or model_field.many_to_one:
                setattr(instance, field, value)
        return super().update(instance, validated_data)


class NestedMappingSerializer(BaseMappingSerializer):
    class Meta:
        fields = "__all__"
        model = None

    def __init__(
        self,
        exclude: list,
        field: models.Field,
        nested_fields: list,
        parent_mapping: dict[str, Any],
        *args: Any,
        **kwargs: Any,
    ) -> None:
        self.exclude = exclude
        self.nested_fields = nested_fields
        self.parent_mapping = parent_mapping
        self.Meta.model = field.related_model
        super().__init__(*args, **kwargs)

    @property
    def mapping(self) -> dict[str, dict[str, Any]]:
        return self.parent_mapping

    @property
    def mapping_fields(self) -> list[str]:
        return self.nested_fields


class MappingSerializer(BaseMappingSerializer):
    _format_prefix = "format"

    class Meta:
        abstract = True
        fields = "__all__"
        model = None

    def __init__(
        self,
        instance: Any = None,
        data: Any = empty,
        **kwargs: Any,
    ) -> None:
        self.instance = instance
        mapped_data = self.map_data(data)
        super().__init__(instance, data=mapped_data, **kwargs)

    def _get_nested_data(self, field_path: list[str], data: Any) -> tuple[Any, bool]:
        field_name = field_path[0]
        if not isinstance(data, dict):
            return None, False
        value = data.get(field_name, None)
        if len(field_path) > 1:
            return self._get_nested_data(field_path[1:], value)
        return value, True

    def _get_data_with_internal_key(
        self, field_path: list[str], parent_data: dict[str, Any] | Any, value: Any
    ) -> dict[str, Any] | Any:
        field_name = field_path[0]
        if len(field_path) > 1:
            nested_data = parent_data.get(field_name, {})
            value = self._get_data_with_internal_key(field_path[1:], nested_data, value)
            if field_name in parent_data:
                nested_data.update(value)
                parent_data.update({field_name: nested_data})
                return parent_data
        return {field_name: value}

    def map_data(self, initial_data: Any) -> Any:
        data: dict[str, Any] = {}
        for external_name, internal_name in self.model_mapping.items():
            external_field_path = external_name.split(self.relation_separator)
            value, found = self._get_nested_data(external_field_path, initial_data)
            if not found:
                continue
            format_func = getattr(self, f"{self._format_prefix}_{internal_name}", None)
            if format_func is not None:
                value = format_func(value)
            internal_field_path = internal_name.split(self.relation_separator)
            if value is None:
                if self.instance is None:
                    continue
                else:
                    try:
                        field = self.model._meta.get_field(internal_field_path[0])
                        if not field.null and field.default != NOT_PROVIDED:
                            continue
                    except FieldDoesNotExist:
                        pass
            data.update(
                self._get_data_with_internal_key(internal_field_path, data, value)
            )
        return data

    @property
    def mapping_fields(self) -> list[str]:
        return list(self.model_mapping.values())

    @property
    def model_mapping(self) -> dict[str, Any]:
        mapping = getattr(self, "mapping", None)
        if mapping is None:
            raise ValueError("Mapping must be set")
        return mapping.get(self.model.__name__.lower(), {})



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from rest_framework import serializers


class CustomFieldQuerySet(models.QuerySet):
    def for_model(self, model: type[models.Model]) -> "CustomFieldQuerySet":
        return self.select_related("content_type").filter(
            content_type__app_label=model._meta.app_label,
            content_type__model=model._meta.model_name,
        )

    def for_type(self, model: type[models.Model]) -> "CustomFieldQuerySet":
        return self.select_related("content_type").filter(
            type_content_type__app_label=model._meta.app_label,
            type_content_type__model=model._meta.model_name,
        )

    def default(self) -> "CustomFieldQuerySet":
        return self.filter(type_id__isnull=True)

    def default_for(self, model: type[models.Model]) -> "CustomFieldQuerySet":
        return self.for_model(model).default()


class FieldType:
    CHAR = "CHAR"
    TEXT = "TEXT"
    DATE = "DATE"
    DATETIME = "DATETIME"
    INTEGER = "INTEGER"
    BOOLEAN = "BOOLEAN"


class CustomField(TimeStampedModel):
    FIELD_TYPES = FieldType

    TYPE_SQL_MAP = {
        FIELD_TYPES.CHAR: "char",
        FIELD_TYPES.TEXT: "text",
        FIELD_TYPES.DATE: "date",
        FIELD_TYPES.DATETIME: "datetime",
        FIELD_TYPES.INTEGER: "integer",
        FIELD_TYPES.BOOLEAN: "boolean",
    }

    TYPE_FIELD_MAP = {
        FIELD_TYPES.CHAR: models.CharField,
        FIELD_TYPES.TEXT: models.TextField,
        FIELD_TYPES.DATE: models.DateField,
        FIELD_TYPES.DATETIME: models.DateTimeField,
        FIELD_TYPES.INTEGER: models.IntegerField,
        FIELD_TYPES.BOOLEAN: models.BooleanField,
    }

    TYPE_SERIALIZER_MAP = {
        FIELD_TYPES.CHAR: serializers.CharField,
        FIELD_TYPES.TEXT: serializers.CharField,
        FIELD_TYPES.DATE: serializers.DateField,
        FIELD_TYPES.DATETIME: serializers.DateTimeField,
        FIELD_TYPES.INTEGER: serializers.IntegerField,
        FIELD_TYPES.BOOLEAN: serializers.BooleanField,
    }

    TYPE_CHOICES = [
        (FIELD_TYPES.CHAR, _("Text (einzeilig)")),
        (FIELD_TYPES.TEXT, _("Text (mehrzeilig)")),
        (FIELD_TYPES.DATE, _("Datum")),
        (FIELD_TYPES.DATETIME, _("Datum und Zeit")),
        (FIELD_TYPES.INTEGER, _("Zahl (Ganzzahl)")),
        (FIELD_TYPES.BOOLEAN, _("Checkbox")),
    ]

    allow_blank = models.BooleanField(
        verbose_name=_("Leeren String erlauben"), default=True
    )
    allow_null = models.BooleanField(
        verbose_name=_("Leere Werte erlauben"), default=True
    )
    choice_field = models.BooleanField(verbose_name=_("Auswahlfeld"), default=False)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    default = models.JSONField(verbose_name=_("Standardwert"), null=True, blank=True)
    editable = models.BooleanField(verbose_name=_("Editierbar"), default=True)
    external_key = models.CharField(
        verbose_name=_("Externer Key"), blank=True, null=True
    )
    field_type = models.CharField(verbose_name=_("Feldtyp"), choices=TYPE_CHOICES)
    hidden = models.BooleanField(verbose_name=_("Ausblenden"), default=False)
    identifier = models.SlugField(max_length=64, unique=True, db_index=True)
    label = models.CharField(verbose_name=_("Name"))
    multiple = models.BooleanField(verbose_name=_("Liste"), default=False)
    order = models.PositiveSmallIntegerField(verbose_name=_("Reihenfolge"), default=0)
    required = models.BooleanField(verbose_name=_("Erforderlich"), default=False)

    type_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.SET_NULL,
        related_name="customfield_set_for_type",
        blank=True,
        null=True,
    )
    type_id = models.PositiveIntegerField(null=True, blank=True)
    type_object = GenericForeignKey(ct_field="type_content_type", fk_field="type_id")

    objects = CustomFieldQuerySet.as_manager()

    class Meta:
        verbose_name = _("Benutzerdefiniertes Feld")
        verbose_name_plural = _("Benutzerdefinierte Felder")
        ordering = ["order", "created"]

    def __str__(self) -> str:
        return f"{self.label}"

    @property
    def choices(self) -> list[tuple[str, str]]:
        from django_features.custom_fields.models import CustomValue

        if not self.choice_field:
            return CustomValue.objects.none()
        return CustomValue.objects.filter(field=self)

    @property
    def output_field(self) -> models.Field:
        output_field = CustomField.TYPE_FIELD_MAP.get(self.field_type)
        if not output_field:
            raise ValueError(f"Unknown field type: {self.field_type}")

        if self.multiple:
            return ArrayField(
                base_field=output_field(blank=self.allow_blank, null=self.allow_null),
                default=self.default,
            )
        return output_field(
            blank=self.allow_blank, null=self.allow_null, default=self.default
        )

    @property
    def serializer_field(self) -> serializers.Field:
        from django_features.custom_fields.fields import ChoiceIdField

        if self.choice_field:
            return ChoiceIdField(field=self)

        serializer_field = self.TYPE_SERIALIZER_MAP.get(self.field_type)
        if serializer_field is None:
            raise ValueError(f"Unknown field type: {self.field_type}")

        params = {"allow_null": self.allow_null}
        if self.default and not self.required:
            params["default"] = self.default
        else:
            params["required"] = self.required

        if self.multiple:
            return serializers.ListField(child=serializer_field(**params), **params)

        return serializer_field(**params)

    @property
    def sql_field(self) -> str:
        sql_field = self.TYPE_SQL_MAP.get(self.field_type)
        if not sql_field:
            raise ValueError(f"Unknown field type: {self.field_type}")
        return sql_field



import json
from typing import Any

import djclick as click
from django.core.management import CommandParser
from django.core.management.base import BaseCommand

from django_features.system_message import models


DEFAULT_SYSTEM_MESSAGE_TYPES = [
    {
        "name": "Info",
        "icon": "information",
    },
    {
        "name": "Warning",
        "icon": "alert",
    },
    {
        "name": "Error",
        "icon": "alert-octagon",
    },
]


class Command(BaseCommand):
    help = "Creates or updates default system message types."

    def add_arguments(self, parser: CommandParser) -> None:
        parser.add_argument(
            "--types",
            type=str,
            help="List of objects with the name and icon of the system message types.",
        )

    def handle(self, types: str, *args: Any, **options: Any) -> None:
        data = DEFAULT_SYSTEM_MESSAGE_TYPES
        if types is not None:
            data = json.loads(types)

        for message_type_data in data:
            name = message_type_data.get("name")

            if not name:
                click.secho(
                    f"ERROR: {message_type_data} has no attribute name", fg="red"
                )

            icon = message_type_data.get("icon")
            info_type, created = models.SystemMessageType.objects.get_or_create(
                name=name
            )
            info_type.icon = icon
            info_type.save()

            click.secho(
                f"INFO: Updated system message type {name} with icon {icon}",
                fg="yellow" if not icon else "green",
            )



from configurations import values


class SystemMessageConfigurationMixin:
    SYSTEM_MESSAGE_PERMISSION = values.Value("")
    ENABLE_SYSTEM_MESSAGE = values.BooleanValue(False)

    @property
    def CONSTANCE_CONFIG(self) -> dict:
        return {
            "SYSTEM_MESSAGE_PERMISSION": (
                self.SYSTEM_MESSAGE_PERMISSION,
                "Django permission to manage system messages.",
                str,
            ),
            "ENABLE_SYSTEM_MESSAGE": (
                self.ENABLE_SYSTEM_MESSAGE,
                "Enables the system info feature.",
                bool,
            ),
        }

    @property
    def CONSTANCE_CONFIG_FIELDSETS(self) -> dict:
        return {
            "System messages": {
                "fields": ("ENABLE_SYSTEM_MESSAGE", "SYSTEM_MESSAGE_PERMISSION"),
                "collapse": True,
            },
        }



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel

from django_features.custom_fields.models.field import CustomField


class CustomValueQuerySet(models.QuerySet):
    def for_model(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.select_related("field", "field__content_type").filter(
            field__content_type__app_label=model._meta.app_label,
            field__content_type__model=model._meta.model_name,
        )

    def for_type(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.select_related("content_type").filter(
            field__type_content_type__app_label=model._meta.app_label,
            field__type_content_type__model=model._meta.model_name,
        )

    def default(self) -> "CustomValueQuerySet":
        return self.filter(field__type_id__isnull=True)

    def default_for(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.for_model(model).default()


class CustomValue(TimeStampedModel):
    field = models.ForeignKey(
        CustomField,
        related_name="values",
        verbose_name=_("Feld"),
        on_delete=models.CASCADE,
    )
    order = models.PositiveIntegerField(_("Reihenfolge"), default=0)
    label = models.CharField(verbose_name=_("Label"), null=True, blank=True)
    value = models.JSONField(verbose_name=_("Wert"), null=True, blank=True)

    objects = CustomValueQuerySet.as_manager()

    class Meta:
        ordering = ["order", "created"]
        verbose_name = _("Benutzerdefinierter Wert")
        verbose_name_plural = _("Benutzerdefinierte Werte")

    def __str__(self) -> str:
        return self.text or ""

    @property
    def text(self) -> str:
        return self.label or str(self.value)
